@@ -2,9 +2,7 @@
 """
 Code needed to run a grammar after it has been compiled.
 """
-import string
 import time
-import operator
 from textwrap import dedent
 from terml.nodes import coerceToTerm, Term, termMaker as t
 from ometa.builder import moduleFromGrammar, writePython
@@ -16,17 +14,10 @@ class ParseError(Exception):
     ?Redo from start
     """
 
-    @property
-    def position(self):
-        return self.args[0]
-
-
-    @property
-    def error(self):
-        return self.args[1]
-
     def __init__(self, input, position, message, trail=None):
         Exception.__init__(self, position, message)
+        self.position = position
+        self.error = message or []
         self.input = input
         self.trail = trail or []
 
@@ -36,8 +27,17 @@ class ParseError(Exception):
             return (self.position, self.error) == (other.position, other.error)
 
 
+    def mergeWith(self, other):
+        """
+        Merges in another error's error and trail.
+        """
+        self.error = list(set(self.error + other.error))
+        self.args = (self.position, self.error)
+        self.trail = other.trail or self.trail or []
+
+
     def formatReason(self):
-        if self.error is None:
+        if not self.error:
             return "Syntax error"
         if len(self.error) == 1:
             if self.error[0][0] == 'message':
@@ -124,24 +124,31 @@ def eof():
     """
     return [("message", "end of input")]
 
-_go = False
+
 def joinErrors(errors):
     """
     Return the error from the branch that matched the most of the input.
     """
-    errors.sort(reverse=True, key=operator.itemgetter(0))
+    if len(errors) == 1:
+        return errors[0]
+
+    highestPos = -1
     results = set()
-    pos = errors[0].position
     trail = None
+
     for err in errors:
-        if pos == err.position:
-            e, trail = err.error, (err.trail or trail)
-            if e is not None:
-                for item in e:
-                        results.add(item)
+        pos = err.position
+        if pos < highestPos:
+            continue
+        elif pos > highestPos:
+            highestPos = pos
+            trail = err.trail or None
+            results = set(err.error)
         else:
-            break
-    return ParseError(errors[0].input,  pos, list(results) or None, trail)
+            trail = err.trail or trail
+            results.update(err.error)
+
+    return ParseError(errors[0].input, highestPos, list(results), trail)
 
 
 class character(str):
@@ -221,7 +228,7 @@ class InputStream(object):
                 data = self.data.__class__('').join(self.data)
             else:
                 data = self.data
-            raise EOFError(data, self.position)
+            raise EOFError(data, self.position + 1)
         return self.data[self.position], self.error
 
     def nullError(self, msg=None):
@@ -299,6 +306,10 @@ class ArgInput(object):
         self.memo = {}
         self.err = parent.nullError()
 
+    @property
+    def position(self):
+        return self.parent.position + 1j
+
     @property
     def data(self):
         return self.parent.data
@@ -385,12 +396,17 @@ class OMetaBase(object):
 
     def considerError(self, error, typ=None):
         if error:
-            if error[0] > self.currentError[0]:
+            newPos = error.position
+            curPos = self.currentError.position
+            if newPos > curPos:
                 self.currentError = error
-            elif error[0] == self.currentError[0]:
-                self.currentError = joinErrors([error, self.currentError])
+            elif newPos == curPos:
+                self.currentError.mergeWith(error)
 
 
+    def _trace(self, src, span, inputPos):
+        pass
+
     def superApply(self, ruleName, *args):
         """
         Apply the named rule as defined on this object's superclass.
@@ -583,7 +599,7 @@ class OMetaBase(object):
         m = self.input
         try:
             fn()
-        except ParseError, e:
+        except ParseError:
             self.input = m
             return True, self.input.nullError()
         else:
@@ -599,9 +615,9 @@ class OMetaBase(object):
                 c, e = self.input.head()
             except EOFError, e:
                 break
-            t = self.input.tail()
+            tl = self.input.tail()
             if c.isspace():
-                self.input = t
+                self.input = tl
             else:
                 break
         return True, e
@@ -717,6 +733,19 @@ class OMetaBase(object):
             self.input = m
             raise e.withMessage(expected("token", tok))
 
+    def label(self, foo, label):
+        """
+        Wrap a function and add label to expected message.
+        """
+        try:
+            val, err = foo()
+            err2 = err.withMessage([("Custom Exception:", label, None)])
+            if self.currentError == err:
+                self.currentError = err2
+            return val, err2
+        except ParseError, e:
+            raise e.withMessage([("Custom Exception:", label, None)])
+
 
     def letter(self):
         """
@@ -800,7 +829,7 @@ class OMetaGrammarBase(OMetaBase):
             start = time.time()
         modname = "pymeta_grammar__" + name
         filename = "/pymeta_generated_code/" + modname + ".py"
-        source = writePython(tree)
+        source = writePython(tree, grammar)
         if TIMING:
             print "Grammar %r generated in %g secs" % (name, time.time() - start)
         return moduleFromGrammar(source, name, modname, filename)
@@ -808,6 +837,7 @@ class OMetaGrammarBase(OMetaBase):
 
     def __init__(self, grammar, *a, **kw):
         OMetaBase.__init__(self, dedent(grammar), *a, **kw)
+        self._spanStart = 0
 
 
     def parseGrammar(self, name):
@@ -822,7 +852,7 @@ class OMetaGrammarBase(OMetaBase):
         self.name = name
         res, err = self.apply("grammar")
         try:
-            x = self.input.head()
+            self.input.head()
         except EOFError:
             pass
         else:
@@ -830,6 +860,12 @@ class OMetaGrammarBase(OMetaBase):
         return res
 
 
+    def startSpan(self):
+        self._spanStart = self.input.position
+
+    def getSpan(self):
+        return (self._spanStart, self.input.position)
+
     def applicationArgs(self, finalChar):
         """
         Collect rule arguments, a list of Python expressions separated by
@@ -853,7 +889,7 @@ class OMetaGrammarBase(OMetaBase):
         else:
             raise self.input.nullError()
 
-    def ruleValueExpr(self, singleLine):
+    def ruleValueExpr(self, singleLine, span=None):
         """
         Find and generate code for a Python expression terminated by a close
         paren/brace or end of line.
@@ -861,26 +897,26 @@ class OMetaGrammarBase(OMetaBase):
         (expr, endchar), err = self.pythonExpr(endChars="\r\n)]")
         # if str(endchar) in ")]" or (singleLine and endchar):
         #     self.input = self.input.prev()
-        return t.Action(expr)
+        return t.Action(expr, span=span)
 
-    def semanticActionExpr(self):
+    def semanticActionExpr(self, span=None):
         """
         Find and generate code for a Python expression terminated by a
         close-paren, whose return value is ignored.
         """
-        val = t.Action(self.pythonExpr(')')[0][0])
+        val = t.Action(self.pythonExpr(')')[0][0], span=span)
         self.exactly(')')
         return val
 
-    def semanticPredicateExpr(self):
+    def semanticPredicateExpr(self, span=None):
         """
         Find and generate code for a Python expression terminated by a
         close-paren, whose return value determines the success of the pattern
         it's in.
         """
-        expr = t.Action(self.pythonExpr(')')[0][0])
+        expr = t.Action(self.pythonExpr(')')[0][0], span=span)
         self.exactly(')')
-        return t.Predicate(expr)
+        return t.Predicate(expr, span=span)
 
 
     def eatWhitespace(self):
@@ -952,6 +988,7 @@ class OMetaGrammarBase(OMetaBase):
                              expected("Python expression"))
         return (''.join(expr).strip(), endchar), e
 
+
     def isTree(self):
         self.tree_target = True
 
@@ -1007,7 +1044,8 @@ class TreeTransformerBase(OMetaBase):
         newargs, e = self.many(self.rule_transform)
         self.end()
         self.input = oldInput
-        return Term(tt.tag, None, tuple(coerceToTerm(a) for a in newargs)), e
+        return Term(tt.tag, None, tuple(coerceToTerm(a) for a in newargs),
+                    tt.span), e
 
     def rule_null(self):
         tt, e = self.rule_anything()
